#include "Car.h"
#include "main.h"

Car::Car(const char* comPort) {
        DCB dcbSerialParams;
        m_hCommPort = CreateFile(comPort,
                GENERIC_WRITE|GENERIC_READ,//access ( read and write)
                0,    //(share) 0:cannot share the COM port
                0,    //security  (None)
                OPEN_EXISTING,// creation : open_existing
                FILE_FLAG_OVERLAPPED,// we want overlapped operation
                0 // no templates file for COM port...
        );

        if(m_hCommPort == NULL)
                throw(1);
        if(m_hCommPort == INVALID_HANDLE_VALUE)
                throw(2);
        if (!GetCommState(m_hCommPort, &dcbSerialParams))
                throw(3);

        dcbSerialParams.DCBlength = sizeof(dcbSerialParams);

        dcbSerialParams.BaudRate = CBR_19200;
        dcbSerialParams.ByteSize = 8;
        dcbSerialParams.StopBits = ONESTOPBIT;
        dcbSerialParams.Parity   = NOPARITY;

        dcbSerialParams.fBinary  = TRUE;

        if (!SetCommState(m_hCommPort, &dcbSerialParams)) {
                throw(4);
        }

        threadReception = new MyThread(this, false);
};

Car::~Car() {
        threadReception->Suspend();
        CloseHandle(m_hCommPort);
        m_hCommPort = (void*)-1;
}

/*
 * initialize the packet
 */
void Car::initPacket() {
        unsigned char i = 0;
        for(; i<8; i++) {
                packet[i] = 0;
        }
}

/*
 * Sends a packet from the class variable 'packet'. Checksum is calculated and
 * set in the packet accordingly.
 */
void Car::sendPacket() {
        DWORD nr=0;
        OVERLAPPED ovlw;
        const unsigned char length = packet[1]+3/*length of data + id, length and checksum*/;
        // Calculate checksum
        packet[length-1] = calculateChecksum(packet, length);

        ovlw.hEvent=CreateEvent(NULL,TRUE,TRUE,NULL);
        WriteFile(m_hCommPort, packet, length, &nr, &ovlw);
        if(!WriteFile(m_hCommPort, packet, length, &nr, &ovlw)) {
                if(GetLastError()!= ERROR_IO_PENDING) {
                        FMain->eReception->Text = "WriteText: "+IntToStr(GetLastError());
                }
        }
}

unsigned char Car::calculateChecksum(const unsigned char* packet,
                const unsigned char length) {
        unsigned char sum = 0;
        unsigned char i = 0;
        for(; i<length-1; i++) {
                sum += packet[i];
        }
        return sum;
}

void Car::singleCommandPacket(Command command) {
        initPacket();
        packet[0] = command;
        sendPacket();
}

void Car::blinkLeft(bool on) {
        if(on) {
                singleCommandPacket(LOnL);
        } else {
                singleCommandPacket(LOffL);
        }
}

void Car::blinkRight(bool on) {
        if(on) {
                singleCommandPacket(LOnR);
        } else {
                singleCommandPacket(LOffR);
        }
}

void Car::blinkWarn(bool on) {
        if(on) {
                singleCommandPacket(LOnW);
        } else {
                singleCommandPacket(LOffW);
        }
}

char* Car::getResponseText(Command response) {
        switch(response) {
        case LOnL:
                return "Turned on Line Change Left";
        case LOffL:
                return "Turned off Line Change Left";
        case LOnR:
                return "Turned on Line Change Right";
        case LOffR:
                return "Turned off Line Change Right";
        case LOnW:
                return "Turned on Warning Lights";
        case LOffW:
                return "Turned off Warning Lights";
        case ASV:
                return "Set the acceleration value";
        case AIV:
                return "Increase acceleration value";
        case ADV:
                return "Decrease acceleration value";
        default:
                return "unknown response";
        }
}

void Car::accelerationSet(unsigned char value) {
        initPacket();
        packet[0] = ASV;/*Command*/
        packet[1] = 1;  /*Length*/
        packet[2] = value; /*Data*/
        sendPacket();
}

void Car::accelerationIncrease(unsigned char value) {
        initPacket();
        packet[0] = AIV;/*Command*/
        packet[1] = 1;  /*Length*/
        packet[2] = value; /*Data*/
        sendPacket();
}

void Car::accelerationDecrease(unsigned char value) {
        initPacket();
        packet[0] = ADV;/*Command*/
        packet[1] = 1;  /*Length*/
        packet[2] = value; /*Data*/
        sendPacket();
}

void Car::sendStateAutomatic(bool on) {
        if(on) {
                singleCommandPacket(SAOn);
        } else {
                singleCommandPacket(SAOff);
        }
}

void Car::sendStateNow() {
        singleCommandPacket(SG);   
}
