/* finite state machine for Line Change/Warning Lights */
enum Light_State { 
  /*-1*/inv=-1,
  /* 0*/NO,
  /* 1*/LEFT,
  /* 2*/RIGHT,
  /* 3*/WARN,
  /* 4*/LEFT_WARN,
  /* 5*/RIGHT_WARN
};
enum Light_State light_state_command_map[6][6] = {
                   /*LOnL       LOffL LOnR        LOffR LOnW        LOffW*/
  /*0 NO*/         { LEFT,      inv,  RIGHT,      inv,  WARN,       inv   },
  /*1 LEFT*/       { inv,       NO,   inv,        inv,  LEFT_WARN,  inv   },
  /*2 RIGHT*/      { inv,       inv,  inv,        NO,   RIGHT_WARN, inv   },
  /*3 WARN*/       { LEFT_WARN, inv,  RIGHT_WARN, inv,  inv,        NO    },
  /*4 LEFT_WARN*/  { inv,       WARN, inv,        inv,  inv,        LEFT  },
  /*5 RIGHT_WARN*/ { inv,       inv,  inv,        WARN, inv,        RIGHT }
};
enum Light_State light_state = NO; // no blinking
enum Light_State last_light_state = NO; // no blinking
void light_fsn() {
  last_light_state = light_state;
  switch(light_state) {
  case NO: // no blinking
    PORTB |= 0x81;
    break;
  case LEFT: // left blinking (P2.15)
    PORTB ^= 0x80;
    PORTB |= 0x01;
    break;
  case RIGHT: // right blinking (P2.8)
    PORTB ^= 0x01;
    PORTB |= 0x80;
    break;
  case WARN: // warning lights (both)
  case LEFT_WARN: // left blinking + warning
  case RIGHT_WARN: // right blinking + warning
    PORTB ^= 0x81;
    break;
  }
}

void main(void) {  
  for(;;) { 
      switch(p_command) {
      /* Lights */
      case LOnL: // Turn on Line Change Left
      case LOffL: // Turn off Line Change Left
      case LOnR: // Turn on Line Change Right
      case LOffR: // Turn off Line Change Right
      case LOnW: // Turn on Warning Lights
      case LOffW: // Turn off Warning Lights
        if(light_state_command_map[light_state][p_command] != inv) {
          light_state = light_state_command_map[light_state][p_command];         
        }
        break;